@@ -3,18 +3,22 @@
 #
 """RAMSES RF - discovery scripts."""
 
+# Beware, none of this is reliable - it is all subject to random change
+# However, these serve as examples how to us eteh other modules
+
 import asyncio
 import json
 import logging
 import re
-from typing import Any, List
+from typing import Any, List, Optional
 
+from .const import __dev_mode__
 from .protocol import RAMSES_CODES, Command, Priority
-from .protocol.const import DEV_REGEX_ANY, DEVICE_TABLE, HGI_DEVICE_ID, NON_DEVICE_ID
+from .protocol.const import DEV_REGEX_ANY, HGI_DEVICE_ID, NON_DEVICE_ID
 from .protocol.exceptions import ExpiredCallbackError
 from .protocol.opentherm import OTB_MSG_IDS
 
-from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
     _0001,
     _0002,
@@ -36,9 +40,11 @@ from .protocol import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
     _10A0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -48,6 +54,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
@@ -69,6 +76,8 @@ from .protocol import (  # noqa: F401, isort: skip
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
     _3B00,
     _3EF0,
@@ -89,6 +98,9 @@ SCAN_XXXX = "scan_xxxx"
 
 DEVICE_ID_REGEX = re.compile(DEV_REGEX_ANY)
 
+QOS_DEFAULT = {"priority": Priority.LOW, "retries": 0}
+QOS_DEFAULT_SCAN = {"priority": Priority.LOW, "retries": 0, "disable_backoff": True}
+
 DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
@@ -96,6 +108,24 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
+def script_decorator(fnc):
+    def wrapper(gwy, *args, **kwargs) -> Optional[Any]:
+
+        gwy.send_cmd(
+            Command._puzzle(message="Script begins:", priority=Priority.HIGH, retries=3)
+        )
+
+        result = fnc(gwy, *args, **kwargs)
+
+        gwy.send_cmd(
+            Command._puzzle(message="Script done.", priority=Priority.LOWEST, retries=3)
+        )
+
+        return result
+
+    return wrapper
+
+
 def spawn_scripts(gwy, **kwargs) -> List[asyncio.Task]:
 
     tasks = []
@@ -113,9 +143,6 @@ def spawn_scripts(gwy, **kwargs) -> List[asyncio.Task]:
         tasks += [gwy._loop.create_task(set_schedule(gwy, *kwargs[SET_SCHED]))]
 
     elif kwargs[EXEC_SCR]:
-        # qos = {"priority": Priority.HIGH, "retries": 3}
-        # gwy.send_cmd(Command._puzzle("00", message="Script: starts...", **qos))
-
         script = SCRIPTS.get(f"{kwargs[EXEC_SCR][0]}")
         if script is None:
             _LOGGER.warning(f"Script: {kwargs[EXEC_SCR][0]}() - unknown script")
@@ -123,9 +150,6 @@ def spawn_scripts(gwy, **kwargs) -> List[asyncio.Task]:
             _LOGGER.info(f"Script: {kwargs[EXEC_SCR][0]}().- starts...")
             tasks += [gwy._loop.create_task(script(gwy, kwargs[EXEC_SCR][1]))]
 
-        qos = {"priority": Priority.LOW, "retries": 3}
-        gwy.send_cmd(Command._puzzle("00", message="Script: ended.", **qos))
-
     gwy._tasks.extend(tasks)
     return tasks
 
@@ -187,7 +211,7 @@ async def get_faults(gwy, ctl_id: str, start=0, limit=0x3F):
     device = gwy._get_device(ctl_id, ctl_id=ctl_id)
 
     try:
-        await device._evo.get_fault_log(start=start, limit=limit)  # 0418
+        await device._evo.get_faultlog(start=start, limit=limit)  # 0418
     except ExpiredCallbackError as exc:
         _LOGGER.error("get_faults(): Function timed out: %s", exc)
 
@@ -224,40 +248,27 @@ async def script_bind_wait(gwy, dev_id: str, code=_2309, idx="00"):
 def script_poll_device(gwy, dev_id) -> List[Any]:
     _LOGGER.warning("poll_device() invoked...")
 
-    qos = {"priority": Priority.LOW, "retries": 0}
-    if "poll_codes" in DEVICE_TABLE.get(dev_id[:2]):
-        codes = DEVICE_TABLE[dev_id[:2]]["poll_codes"]
-    else:
-        codes = [_0016, _1FC9]
-
     tasks = []
 
-    for code in codes:
-        cmd = Command(RQ, code, "00", dev_id, **qos)
-        tasks.append(gwy._loop.create_task(periodic(gwy, cmd, count=0)))
-
-        cmd = Command(RQ, code, "0000", dev_id, **qos)
+    for code in (_0016, _1FC9):
+        cmd = Command(RQ, code, "00", dev_id, **QOS_DEFAULT)
         tasks.append(gwy._loop.create_task(periodic(gwy, cmd, count=0)))
 
     gwy._tasks.extend(tasks)
     return tasks
 
 
+@script_decorator
 async def script_scan_disc(gwy, dev_id: str):
     _LOGGER.warning("scan_quick() invoked...")
 
-    qos = {"priority": Priority.HIGH, "retries": 3}
-    gwy.send_cmd(Command._puzzle("00", message="disc scan: begins...", **qos))
-
-    gwy._get_device(dev_id)._discover()  # discover_flag=DISCOVER_ALL)
+    gwy._get_device(dev_id)._discover()  # discover_flag=Discover.ALL)
 
 
+@script_decorator
 async def script_scan_full(gwy, dev_id: str):
     _LOGGER.warning("scan_full() invoked - expect a lot of Warnings")
 
-    qos = {"priority": Priority.HIGH, "retries": 3}
-    gwy.send_cmd(Command._puzzle("00", message="full scan: begins...", **qos))
-
     qos = {"priority": Priority.DEFAULT, "retries": 5}
     gwy.send_cmd(Command(RQ, _0016, "0000", dev_id, **qos))
 
@@ -280,7 +291,7 @@ async def script_scan_full(gwy, dev_id: str):
         #         gwy.send_cmd(Command(W_, code, f"{zone_idx}03", dev_id, **qos))
 
         elif code == _0404:
-            gwy.send_cmd(Command.get_schedule_fragment(dev_id, "HW", 0, **qos))
+            gwy.send_cmd(Command.get_schedule_fragment(dev_id, "HW", 0, 0, **qos))
             gwy.send_cmd(Command.get_schedule_fragment(dev_id, "00", 0, 0, **qos))
 
         elif code == _0418:
@@ -316,18 +327,16 @@ async def script_scan_full(gwy, dev_id: str):
         gwy.send_cmd(Command(RQ, code, "0000", dev_id, **qos))
 
 
+@script_decorator
 async def script_scan_hard(gwy, dev_id: str):
     _LOGGER.warning("scan_hard() invoked - expect some Warnings")
 
-    qos = {"priority": Priority.HIGH, "retries": 3}
-    gwy.send_cmd(Command._puzzle("00", message="hard scan: begins...", **qos))
-
-    qos = {"priority": Priority.LOW, "retries": 0}
     for code in range(0x4000):
-        gwy.send_cmd(Command(RQ, f"{code:04X}", "0000", dev_id, **qos))
+        gwy.send_cmd(Command(RQ, f"{code:04X}", "0000", dev_id, **QOS_DEFAULT_SCAN))
         await asyncio.sleep(1)
 
 
+@script_decorator
 async def script_scan_001(gwy, dev_id: str):
     _LOGGER.warning("scan_001() invoked - expect a lot of nonsense")
 
@@ -337,53 +346,103 @@ async def script_scan_001(gwy, dev_id: str):
         gwy.send_cmd(Command(RQ, _000E, f"{idx:02X}00C8", dev_id, **qos))
 
 
+@script_decorator
 async def script_scan_002(gwy, dev_id: str):
     _LOGGER.warning("scan_002() invoked - expect a lot of nonsense")
 
     # Two modes, I and W & Two headers zz00 and zz
     message = "0000" + "".join(f"{ord(x):02X}" for x in "Hello there.") + "00"
-    qos = {"priority": Priority.LOW, "retries": 0}
+
     [
-        gwy.send_cmd(Command(W_, f"{c:04X}", message, dev_id, **qos))
+        gwy.send_cmd(Command(W_, f"{c:04X}", message, dev_id, **QOS_DEFAULT))
         for c in range(0x4000)
         if c not in RAMSES_CODES
     ]
 
 
-async def script_scan_003(gwy, dev_id: str):
-    _LOGGER.warning("scan_003() invoked - expect a lot of nonsense")
-
-    qos = {"priority": Priority.LOW, "retries": 0}
-    for msg_id in range(0x100):
-        gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **qos))
-
-
 async def script_scan_004(gwy, dev_id: str):
     _LOGGER.warning("scan_004() invoked - expect a lot of nonsense")
 
-    qos = {"priority": Priority.LOW, "retries": 0}
-
-    cmd = Command.get_dhw_mode(dev_id, **qos)
+    cmd = Command.get_dhw_mode(dev_id, **QOS_DEFAULT)
 
     return gwy._loop.create_task(periodic(gwy, cmd, count=0, interval=5))
 
 
-async def script_scan_005(gwy, dev_id: str):
-    _LOGGER.warning("scan_005(otb, full) invoked - expect a lot of nonsense")
+@script_decorator
+async def script_scan_otb(gwy, dev_id: str):
+    _LOGGER.warning("script_scan_otb_full invoked - expect a lot of nonsense")
 
     qos = {"priority": Priority.LOW, "retries": 1}
-
     for msg_id in OTB_MSG_IDS:
         gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **qos))
 
 
-async def script_scan_006(gwy, dev_id: str):
-    _LOGGER.warning("scan_006(otb, hard) invoked - expect a lot of nonsense")
-
-    qos = {"priority": Priority.LOW, "retries": 0}
+@script_decorator
+async def script_scan_otb_hard(gwy, dev_id: str):
+    _LOGGER.warning("script_scan_otb_hard invoked - expect a lot of nonsense")
 
     for msg_id in range(0x80):
-        gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **qos))
+        gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **QOS_DEFAULT_SCAN))
+
+
+@script_decorator
+async def script_scan_otb_map(gwy, dev_id: str):  # Tested only upon a R8820A
+    _LOGGER.warning("script_scan_otb_map invoked - expect a lot of nonsense")
+
+    RAMSES_TO_OPENTHERM = {
+        _22D9: "01",  # boiler setpoint        / ControlSetpoint
+        _3EF1: "11",  # rel. modulation level  / RelativeModulationLevel
+        _1300: "12",  # cv water pressure      / CHWaterPressure
+        _3200: "19",  # boiler output temp     / BoilerWaterTemperature
+        _1260: "1A",  # dhw temp               / DHWTemperature
+        _1290: "1B",  # outdoor temp           / OutsideTemperature
+        _3210: "1C",  # boiler return temp     / ReturnWaterTemperature
+        _10A0: "38",  # dhw params["setpoint"] / DHWSetpoint
+        _1081: "39",  # max ch setpoint        / MaxCHWaterSetpoint
+    }
+
+    for code, msg_id in RAMSES_TO_OPENTHERM.items():
+        gwy.send_cmd(Command(RQ, code, "00", dev_id, **QOS_DEFAULT))
+        gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **QOS_DEFAULT))
+
+
+@script_decorator
+async def script_scan_otb_ramses(gwy, dev_id: str):  # Tested only upon a R8820A
+    _LOGGER.warning("script_scan_otb_ramses invoked - expect a lot of nonsense")
+
+    CODES = (
+        _042F,
+        _10E0,  # device_info
+        _10E1,  # device_id
+        "1FD0",
+        "2400",
+        "2401",
+        "2410",
+        "2420",
+        _1300,  # cv water pressure      / CHWaterPressure
+        _1081,  # max ch setpoint        / MaxCHWaterSetpoint
+        _10A0,  # dhw params["setpoint"] / DHWSetpoint
+        _22D9,  # boiler setpoint        / ControlSetpoint
+        _1260,  # dhw temp               / DHWTemperature
+        _1290,  # outdoor temp           / OutsideTemperature
+        _3200,  # boiler output temp     / BoilerWaterTemperature
+        _3210,  # boiler return temp     / ReturnWaterTemperature
+        "0150",
+        "12F0",  # DHW flow rate?
+        "1098",
+        "10B0",
+        "3221",
+        "3223",
+        _3EF0,  # rel. modulation level  / RelativeModulationLevel (also, below)
+        _3EF1,  # rel. modulation level  / RelativeModulationLevel
+    )  # excl. 3220
+
+    # 3EF0 also includes:
+    #  - boiler status        /
+    #  - ch setpoint          /
+    #  - max. rel. modulation /
+
+    [gwy.send_cmd(Command(RQ, c, "00", dev_id, **QOS_DEFAULT)) for c in CODES]
 
 
 SCRIPTS = {
