@@ -125,6 +125,7 @@ QOS_TABLE = {  # priority, retries, timeout, disable_backoff, c.f. QOS_TX_DEFAUL
     f"{RQ}/{_3220}": (Priority.DEFAULT, 1, td(seconds=1), True),
 }  # The long timeeout for the OTB is for total RTT to slave (boiler)
 
+
 DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
@@ -132,23 +133,30 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def validate_command(has_zone=None):
+def validate_api_params(has_zone=None):
     """Decorator to protect the engine from any invalid command constructors.
 
-    Additionally, validate/normalise some command arguments (e.g. 'HW' becomes 'FA').
+    Additionally, validate/normalise some command arguments (e.g. 'HW' becomes 0xFA).
+    NB: The zone_idx (domain_id) is converted to an integer, but payloads use strings
+    such as f"{zone_idx}:02X".
     """
 
     def _wrapper(fcn, *args, **kwargs) -> Any:
+        _LOGGER.debug(f"Calling: {fcn.__name__}({args}, {kwargs})")
         try:
             return fcn(*args, **kwargs)
         except (
+            ArithmeticError,  # incl. ZeroDivisionError,
             AssertionError,
             AttributeError,
-            LookupError,
+            IndexError,
+            LookupError,  # incl. IndexError, KeyError
+            NameError,  # incl. UnboundLocalError
+            RuntimeError,  # incl. RecursionError
             TypeError,
             ValueError,
         ) as exc:
-            _LOGGER.exception(f"validate_command(): {exc}")
+            _LOGGER.exception(f"{fcn.__name__}({args}, {kwargs}): {exc}")
 
     def validate_zone_idx(zone_idx) -> int:
         if isinstance(zone_idx, str):
@@ -162,8 +170,10 @@ def validate_command(has_zone=None):
         @functools.wraps(fcn)
         def wrapper(cls, dst_id, *args, **kwargs) -> Any:
 
-            if "zone_idx" in kwargs:
+            if "zone_idx" in kwargs:  # Cmd.get_relay_demand()
                 kwargs["zone_idx"] = validate_zone_idx(kwargs["zone_idx"])
+            if "domain_id" in kwargs:
+                kwargs["domain_id"] = validate_zone_idx(kwargs["domain_id"])
 
             return _wrapper(fcn, cls, dst_id, *args, **kwargs)
 
@@ -174,6 +184,8 @@ def validate_command(has_zone=None):
         def wrapper(cls, ctl_id, zone_idx, *args, **kwargs) -> Any:
 
             zone_idx = validate_zone_idx(zone_idx)
+            if "domain_id" in kwargs:
+                kwargs["domain_id"] = validate_zone_idx(kwargs["domain_id"])
 
             return _wrapper(fcn, cls, ctl_id, zone_idx, *args, **kwargs)
 
@@ -370,13 +382,14 @@ class Command(PacketBase):
         return (self._priority, self._dtm) < (other._priority, other._dtm)
 
     @classmethod  # constructor for RQ/1F41
-    @validate_command()
+    @validate_api_params()
     def get_dhw_mode(cls, ctl_id: str, **kwargs):
         """Constructor to get the mode of the DHW (c.f. parser_1f41)."""
+
         return cls(RQ, _1F41, "00", ctl_id, **kwargs)
 
     @classmethod  # constructor for W_/1F41
-    @validate_command()
+    @validate_api_params()
     def set_dhw_mode(
         cls,
         ctl_id: str,
@@ -410,13 +423,14 @@ class Command(PacketBase):
         return cls(W_, _1F41, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/10A0
-    @validate_command()
+    @validate_api_params()
     def get_dhw_params(cls, ctl_id: str, **kwargs):
         """Constructor to get the params of the DHW (c.f. parser_10a0)."""
+
         return cls(RQ, _10A0, "00", ctl_id, **kwargs)
 
     @classmethod  # constructor for W_/10A0
-    @validate_command()
+    @validate_api_params()
     def set_dhw_params(
         cls,
         ctl_id: str,
@@ -447,19 +461,21 @@ class Command(PacketBase):
         return cls(W_, _10A0, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/1260
-    @validate_command()
+    @validate_api_params()
     def get_dhw_temp(cls, ctl_id: str, **kwargs):
         """Constructor to get the temperature of the DHW sensor (c.f. parser_10a0)."""
+
         return cls(RQ, _1260, "00", ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/1030
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_mix_valve_params(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the mix valve params of a zone (c.f. parser_1030)."""
+
         return cls(RQ, _1030, f"{zone_idx:02X}00", ctl_id, **kwargs)  # TODO: needs 00?
 
     @classmethod  # constructor for W/1030
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def set_mix_valve_params(
         cls,
         ctl_id: str,
@@ -491,21 +507,23 @@ class Command(PacketBase):
         return cls(W_, _1030, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/3220
-    @validate_command()
+    @validate_api_params()
     def get_opentherm_data(cls, dev_id: str, msg_id: Union[int, str], **kwargs):
         """Constructor to get (Read-Data) opentherm msg value (c.f. parser_3220)."""
+
         msg_id = msg_id if isinstance(msg_id, int) else int(msg_id, 16)
         payload = f"0080{msg_id:02X}0000" if parity(msg_id) else f"0000{msg_id:02X}0000"
         return cls(RQ, _3220, payload, dev_id, **kwargs)
 
     @classmethod  # constructor for RQ/0008
-    @validate_command()  # has_zone=Optional
+    @validate_api_params()  # has_zone=Optional
     def get_relay_demand(cls, dev_id: str, zone_idx: Union[int, str] = "00", **kwargs):
         """Constructor to get the demand of a relay/zone (c.f. parser_0008)."""
-        return cls(RQ, _0008, zone_idx, dev_id, **kwargs)
+
+        return cls(RQ, _0008, f"{zone_idx:02X}", dev_id, **kwargs)
 
     @classmethod  # constructor for RQ/0404
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_schedule_fragment(
         cls,
         ctl_id: str,
@@ -526,26 +544,29 @@ class Command(PacketBase):
         return cls(RQ, _0404, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/0100
-    @validate_command()
+    @validate_api_params()
     def get_system_language(cls, ctl_id: str, **kwargs):
         """Constructor to get the language of a system (c.f. parser_0100)."""
+
         return cls(RQ, _0100, "00", ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/0418
-    @validate_command()
+    @validate_api_params()
     def get_system_log_entry(cls, ctl_id: str, log_idx: int, **kwargs):
         """Constructor to get a log entry from a system (c.f. parser_0418)."""
+
         log_idx = log_idx if isinstance(log_idx, int) else int(log_idx, 16)
         return cls(RQ, _0418, f"{log_idx:06X}", ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/2E04
-    @validate_command()
+    @validate_api_params()
     def get_system_mode(cls, ctl_id: str, **kwargs):
         """Constructor to get the mode of a system (c.f. parser_2e04)."""
+
         return cls(RQ, _2E04, "FF", ctl_id, **kwargs)
 
     @classmethod  # constructor for W/2E04
-    @validate_command()
+    @validate_api_params()
     def set_system_mode(cls, ctl_id: str, system_mode, until=None, **kwargs):
         """Constructor to set/reset the mode of a system (c.f. parser_2e04)."""
 
@@ -577,13 +598,14 @@ class Command(PacketBase):
         return cls(W_, _2E04, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/313F
-    @validate_command()
+    @validate_api_params()
     def get_system_time(cls, ctl_id: str, **kwargs):
         """Constructor to get the datetime of a system (c.f. parser_313f)."""
+
         return cls(RQ, _313F, "00", ctl_id, **kwargs)
 
     @classmethod  # constructor for W/313F
-    @validate_command()
+    @validate_api_params()
     def set_system_time(cls, ctl_id: str, datetime, **kwargs):
         """Constructor to set the datetime of a system (c.f. parser_313f)."""
         #  W --- 30:185469 01:037519 --:------ 313F 009 0060003A0C1B0107E5
@@ -591,15 +613,16 @@ class Command(PacketBase):
         return cls(W_, _313F, f"006000{dtm_to_hex(datetime)}", ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/1100
-    @validate_command()
+    @validate_api_params()
     def get_tpi_params(cls, dev_id: str, domain_id=None, **kwargs):
         """Constructor to get the TPI params of a system (c.f. parser_1100)."""
+
         if domain_id is None:
             domain_id = "00" if dev_id[:2] == "13" else "FC"
         return cls(RQ, _1100, domain_id, dev_id, **kwargs)
 
     @classmethod  # constructor for W/1100
-    @validate_command()
+    @validate_api_params()
     def set_tpi_params(
         cls,
         ctl_id: str,
@@ -632,13 +655,14 @@ class Command(PacketBase):
         return cls(W_, _1100, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/000A
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_zone_config(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the config of a zone (c.f. parser_000a)."""
+
         return cls(RQ, _000A, f"{zone_idx:02X}00", ctl_id, **kwargs)  # TODO: needs 00?
 
     @classmethod  # constructor for W/000A
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def set_zone_config(
         cls,
         ctl_id: str,
@@ -674,13 +698,14 @@ class Command(PacketBase):
         return cls(W_, _000A, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/2349
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_zone_mode(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the mode of a zone (c.f. parser_2349)."""
+
         return cls(RQ, _2349, f"{zone_idx:02X}00", ctl_id, **kwargs)  # TODO: needs 00?
 
     @classmethod  # constructor for W/2349
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def set_zone_mode(
         cls,
         ctl_id: str,
@@ -726,38 +751,43 @@ class Command(PacketBase):
         return cls(W_, _2349, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/0004
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_zone_name(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the name of a zone (c.f. parser_0004)."""
+
         return cls(RQ, _0004, f"{zone_idx:02X}00", ctl_id, **kwargs)  # TODO: needs 00?
 
     @classmethod  # constructor for W/0004
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def set_zone_name(cls, ctl_id: str, zone_idx: Union[int, str], name: str, **kwargs):
         """Constructor to set the name of a zone (c.f. parser_0004)."""
+
         payload = f"{zone_idx:02X}00{str_to_hex(name)[:24]:0<40}"  # TODO: check 12/24?
         return cls(W_, _0004, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for W/2309
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def set_zone_setpoint(
         cls, ctl_id: str, zone_idx: Union[int, str], setpoint: float, **kwargs
     ):
         """Constructor to set the setpoint of a zone (c.f. parser_2309)."""
         #  W --- 34:092243 01:145038 --:------ 2309 003 0107D0
+
         payload = f"{zone_idx:02X}{temp_to_hex(setpoint)}"
         return cls(W_, _2309, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/30C9
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_zone_temp(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the current temperature of a zone (c.f. parser_30c9)."""
+
         return cls(RQ, _30C9, f"{zone_idx:02X}", ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/12B0
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def get_zone_window_state(cls, ctl_id: str, zone_idx: Union[int, str], **kwargs):
         """Constructor to get the openwindow state of a zone (c.f. parser_12b0)."""
+
         return cls(RQ, _12B0, f"{zone_idx:02X}", ctl_id, **kwargs)
 
     @classmethod  # generic constructor
@@ -796,7 +826,7 @@ class Command(PacketBase):
         return cmd
 
     @classmethod  # constructor for RP/3EF1 (TODO: & I/3EF1?)
-    @validate_command()
+    @validate_api_params()
     def put_actuator_cycle(
         cls,
         src_id: str,
@@ -820,7 +850,7 @@ class Command(PacketBase):
         return cls.packet(RP, _3EF1, payload, addr0=src_id, addr1=dst_id, **kwargs)
 
     @classmethod  # constructor for I/3EF0
-    @validate_command()
+    @validate_api_params()
     def put_actuator_state(cls, dev_id: str, mod_level: float, **kwargs):
         """Constructor to announce the modulation level of an actuator (3EF0).
 
@@ -862,7 +892,7 @@ class Command(PacketBase):
         )
 
     @classmethod  # constructor for I/0002
-    @validate_command()
+    @validate_api_params()
     def put_outdoor_temp(cls, dev_id: str, temperature: float, **kwargs):
         """Constructor to announce the current temperature of an outdoor sensor (0002).
 
@@ -875,7 +905,7 @@ class Command(PacketBase):
         return cls.packet(I_, _0002, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
     @classmethod  # constructor for I/30C9
-    @validate_command()
+    @validate_api_params()
     def put_sensor_temp(cls, dev_id: str, temperature: float, **kwargs):
         """Constructor to announce the current temperature of a thermostat (3C09).
 
@@ -890,7 +920,7 @@ class Command(PacketBase):
         return cls.packet(I_, _30C9, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
     @classmethod  # constructor for W/0404
-    @validate_command(has_zone=True)
+    @validate_api_params(has_zone=True)
     def put_schedule_fragment(
         cls,
         ctl_id: str,
@@ -902,7 +932,7 @@ class Command(PacketBase):
     ):
         """Constructor to put a zone schedule fragment (c.f. parser_0404).
 
-        Usually a zone, but will be the DHW schedule if zone_idx == 0xFA, 'FA', or 'HW'..
+        Usually a zone, but will be the DHW schedule if zone_idx == 0xFA, 'FA', or 'HW'.
         """
 
         header = "00230008" if zone_idx == 0xFA else f"{zone_idx:02X}200008"
@@ -940,7 +970,7 @@ class Command(PacketBase):
 
 
 # A convenience dict
-_COMMANDS = {
+CODE_API_MAP = {
     f"{I_}/{_0002}": Command.put_outdoor_temp,
     f"{RQ}/{_0004}": Command.get_zone_name,
     f"{W_}/{_0004}": Command.set_zone_name,
