@@ -24,6 +24,11 @@ from .const import (
     ATTR_DHW_VALVE,
     ATTR_DHW_VALVE_HTG,
     ATTR_HTG_CONTROL,
+    BOOST_TIMER,
+    FAN_MODE,
+    FAN_MODES,
+    HEATER_MODE,
+    HEATER_MODES,
     SYSTEM_MODE,
     ZONE_MODE,
 )
@@ -121,9 +126,9 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def parser_decorator(func):
+def parser_decorator(fnc):  # TODO: remove
     def wrapper(*args, **kwargs) -> Optional[Any]:
-        return func(*args, **kwargs)
+        return fnc(*args, **kwargs)
 
     return wrapper
 
@@ -173,10 +178,18 @@ def parser_0001(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # sensor_weather
 def parser_0002(payload, msg) -> Optional[dict]:
-    # I --- 03:125829 --:------ 03:125829 0002 004 03020105  # seems to be faked
+    # seen with: 03:125829, 03:196221, 03:196196, 03:052382, 03:201498, 03:201565:
+    #  I 000 03:201565 --:------ 03:201565 0002 004 03020105  # no zone_idx, domain_id
 
-    assert msg.len == 4
+    # is it CODE_IDX_COMPLEX:
+    #  - 02...... for outside temp?
+    #  - 03...... for other stuff?
 
+    if msg.src.type == "03":  # payload[2:] == "03", DEX
+        assert payload == "03020105"
+        return {"_unknown": payload}
+
+    # if payload[6:] == "02":  # msg.src.type == "17":
     return {
         "temperature": temp_from_hex(payload[2:6]),
         "_unknown": payload[6:],
@@ -207,8 +220,10 @@ def parser_0005(payload, msg) -> Optional[dict]:  # TODO: needs a cleanup
     #  I --- 34:064023 --:------ 34:064023 0005 012 000A0000-000F0000-00100000
 
     def _parser(seqx) -> dict:
-        if msg.src.type == "02":  # seqx[2:4] == _0005_ZONE.UFH:
+        if msg.src.type == "02":  # DEX, or use: seqx[2:4] == _0005_ZONE.UFH:
             zone_mask = flag8(seqx[6:8], lsb=True)
+        elif msg.len == 3:  # ATC928G1000 - 1st gen monochrome model, max 8 zones
+            zone_mask = flag8(seqx[4:6], lsb=True)
         else:
             zone_mask = flag8(seqx[4:6], lsb=True) + flag8(seqx[6:8], lsb=True)
         return {
@@ -724,8 +739,9 @@ def parser_10e0(payload, msg) -> Optional[dict]:
     try:
         if msg.src.type == "01":  # DEX
             assert payload[2:20] in (
-                "0002FF0119FFFFFFFF",  # EvoTouch Colour
-                "0002FF0163FFFFFFFF",  # Evo Color
+                "0002FF0119FFFFFFFF",  # ATC928-G3-0xx Evo Mk3 - EvoTouch Colour (WiFi, 12 zones)
+                "0002FF0163FFFFFFFF",  # ATP928-G2-080 Evo Mk2 - Color (no WiFi)
+                "0002FFFF17FFFFFFFF",  # ATC928-G1-000 Evo Mk1 - Monochrone (?prototype, 8 zones)
             ), payload[2:20]
         elif msg.src.type == "02":  # DEX
             assert payload[2:20] in (
@@ -1199,23 +1215,22 @@ def parser_22d9(payload, msg) -> Optional[dict]:
 
 
 @parser_decorator  # switch_mode
-def parser_22f1(payload, msg) -> Optional[dict]:
-    from ..devices import FanSwitch  # FIXME: remove cyclic reference
-
+def parser_22f1(payload, msg) -> Optional[dict]:  # FIXME
     # 11:42:43.149 081  I 051 --:------ --:------ 49:086353 22F1 003 000304
     # 11:42:49.587 071  I 052 --:------ --:------ 49:086353 22F1 003 000404
     # 11:42:49.685 072  I 052 --:------ --:------ 49:086353 22F1 003 000404
     # 11:42:49.784 072  I 052 --:------ --:------ 49:086353 22F1 003 000404
+
     # assert payload[:2] == "00", payload[:2]  # has no domain
     assert int(payload[2:4], 16) <= int(payload[4:], 16), "step_idx not <= step_max"
     # assert payload[4:] in ("04", "0A"), payload[4:]
 
     bitmap = int(payload[2:4], 16)
 
-    if bitmap in FanSwitch.FAN_MODES:
-        _action = {FanSwitch.FAN_MODE: FanSwitch.FAN_MODES[bitmap]}
+    if bitmap in FAN_MODES:
+        _action = {FAN_MODE: FAN_MODES[bitmap]}
     elif bitmap in {9, 10}:  # 00010001, 00010010
-        _action = {FanSwitch.HEATER_MODE: FanSwitch.HEATER_MODES[bitmap]}
+        _action = {HEATER_MODE: HEATER_MODES[bitmap]}
     else:
         _action = {}
 
@@ -1235,7 +1250,7 @@ def parser_22f3(payload, msg) -> Optional[dict]:
 
     if msg.len >= 3:
         result = {
-            "mode": "boost_timer",  # payload[2:4]
+            "mode": BOOST_TIMER,  # payload[2:4]
             "minutes": int(payload[4:6], 16),
         }
 
@@ -1324,6 +1339,11 @@ def parser_2349(payload, msg) -> Optional[dict]:
     return result
 
 
+@parser_decorator  # unknown
+def parser_2389(payload, msg) -> Optional[dict]:
+    return {"_unknown": temp_from_hex(payload[2:6])}
+
+
 @parser_decorator  # hometronics _state (of unknwon)
 def parser_2d49(payload, msg) -> dict:
     assert payload[2:] in ("0000", "C800"), payload[2:]  # would "FFFF" mean N/A?
